#!/usr/bin/env tclsh
source where.tcl

set F(user) {"Desired user name, format user@domain"
            1 text 30
            }
set F(givenname) {"Your given (first) name"  3 text_name 30}
set F(surname)   {"Your surname (last name)" 4 text_name 30}

# Create a new user.
# form for creating new user
proc NewUserForm {{user ""} {email ""}} {
  global F
  h2 "Create new user"
  set V(user)  $user
  set V(email) $email
  cgi_form UserNew {
    tableform "=Create" F V
  }
}


cgi_eval {
  global CINEMA
  set level [auth UserNew "New User" 0]
  cgi_head {
    cgi_title "CINEMA: Creating New User"
  }
  #validate_user_pw_script

  # check for CINEMA global here after auth.
  if {$CINEMA(mode) == "remote"} {
    set F(email) {"Your email address"
                 2 text_email 30
                 }
  } 

  cgi_body {
    import_default user
    import_default pass
    import_default email
    import_default surname
    import_default givenname
    banner

    # if empty user, show form
    if {$user == ""} {
      NewUserForm
      done
    }

    # add domain if not given
    set uh [split $user "@"]
    if {$user != "" && [llength $CINEMA(server_name)] == 1 && [lindex $uh 1] == ""} {
      set user "[lindex $uh 0]@$CINEMA(server_name)"
    }

    # check syntax of user name
    if {[set msg [checkuser $user]] != ""} {
      p $msg
      NewUserForm $user $email
      done
    }

    # check if domain is valid
    set uh [split $user "@"]
    if {[lsearch $CINEMA(server_name) [lindex $uh 1]] < 0} {
      p "Domain name must be [bold $CINEMA(server_name)]."
      NewUserForm $user $email
      done
    }

    # if not defined, try user name as email
    if {$email == ""} {
      set email $user
    }

    # check email address
    if {[set msg [CheckEmail $email]] != ""} {
      p $msg
      NewUserForm $user $email
      done
    }

    # check if user exists already
    set olduser [sql "SELECT user FROM put WHERE user='$user'"] 
    if {$olduser != ""} {
      p "The name [bold $user] is already taken. Please choose another
name or [url login [cgi index [cgi_set user $user]]]."
      NewUserForm
      done
    }

    # if on local system, make sure user exists
    if {$CINEMA(mode) == "local" && [localuser $user 2] < 1} {
      NewUserForm $user $email
      done
    }

    # obtain default values from default@domain entry if no existing user
    h2 "Creating new user $user"

    p "Reading default user [bold default@[domain $user]]."
    set default [sql "SELECT realm,auth,algorithm,sip_methods,sip_groups,gwclass,um_timeout,max_msgsize_kb,busy,noresponse,message_template FROM put WHERE user='default@[domain $user]'"]
    maplist [lindex $default 0] realm auth algorithm sip_methodList sip_groupList gwclass um_timeout max_msgsize_kb busy noresponse message_template
    set hash_value ""

    set pass [randomword 3]
    set realm $CINEMA(realm)
    set hash [hash $user $realm $pass]
    if {$gwclass == ""} {
      set gwclass [gwclass_default $user]
    }

    # insert into address book
    if {$CINEMA(mode) == "remote"} {
       set q [tableforminput F {surname givenname email}]
    } else {
       # For mode == local, email and user are same.
       set q [tableforminput F {surname givenname}]
       lappend q "email='$user'"
    }
    sql "INSERT person SET [join $q ,]"

    # insert into Primary User Table, making link to address book
    sql "INSERT put SET \
      user='$user',\
      realm='$realm',\
      auth='$auth',\
      hash_value='$hash',\
      algorithm='$algorithm',\
      sip_groups='$sip_groupList',\
      gwclass='$gwclass',\
      sip_methods='$sip_methodList',\
      message_template='$message_template',\
      um_timeout='$um_timeout',\
      max_msgsize_kb='$max_msgsize_kb',\
      noresponse='$noresponse',\
      busy='$busy',\
      person=LAST_INSERT_ID()"

    if {[lsearch $sip_groupList voicemail] >= 0} {
      vmailadd $user
    }

    # Send email to new user
    lappend substitutions [list user      $user]
    lappend substitutions [list domain    [domain $user]]
    lappend substitutions [list password  $pass]
    lappend substitutions [list CINEMA    $CINEMA(root)]
    lappend substitutions [list realm     $realm]

    set msg {Your new CINEMA password for $user
      in $domain is "$password".
      You can login at $CINEMA.
      The realm is "$realm".}
    email::email $email "" "" "" "Your CINEMA registration" $msg $substitutions
    p "Email with the password for [bold $user] has been sent to [bold $email]."

    p "[url Login [cgi index [cgi_cgi for $user]] target=_top] to your account."

    sql disconnect
  }
}
